@@ -1,7 +1,7 @@
 
 package MooseX::Timestamp;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -13,6 +13,7 @@ MooseX::Timestamp - simple timestamp type for Moose
 
  print timestamp;          # 2007-12-06 23:15:42
  print timestamp 0;        # 1970-01-01 12:00:00
+ print timestamp 0.0001;   # 1970-01-01 12:00:00.0001
  print timestamp gmtime 0; # 1970-01-01 00:00:00
 
  use POSIX qw(strftime);
@@ -35,7 +36,10 @@ MooseX::Timestamp - simple timestamp type for Moose
 
 =head1 DESCRIPTION
 
-Tired of bulky date modules?  Don't put up with them any longer.
+This module provides a timestamp type as a Str subtype for Moose.
+This is a much more lightweight format than, say, L<DateTime>, with
+the disadvantage that it does not support native operations on the
+dates.
 
 This module provides floating dates on the Gregorian calendar without
 much code.  It operates in (one or two particular variants of)
@@ -59,7 +63,7 @@ use Carp;
 subtype Timestamp
     => as Str
     => where {
-	    m{^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$} and
+	    m{^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}(\.\d+)?$} and
 		    eval { valid_posixtime(posixtime($_)) };
     };
 
@@ -70,10 +74,26 @@ sub timestamp {
 	}
 	if ( @_ == 1 ) {
 		my $time = shift;
-		@_ = localtime($time);
+		my $frac = $time - int($time);
+		@_ = localtime(int($time));
+		$_[0] += $frac;
 	}
 	valid_posixtime(@_);
-	strftime("%Y-%m-%d %H:%M:%S", @_ );
+	if ( int($_[0]) == $_[0] ) {
+		strftime("%Y-%m-%d %H:%M:%S", @_ ),
+	}
+	else {
+		# microseconds only.  Any more and you start seeing FP
+		# precision weirdness a lot more than you'd expect.
+		my $sec = sprintf("%.6f", $_[0]);
+		$sec =~ s{0+$}{};
+		join(
+			"",
+			strftime("%Y-%m-%d %H:%M:", @_ ),
+			($_[0]<10)?("0"):(),
+			$sec,
+		       );
+	}
 }
 
 my @short = qw(0 1 0 1 0 1 0 0 1 0 1 0);
@@ -89,7 +109,7 @@ sub valid_posixtime {
 	croak "invalid hour $lt[2]" if $lt[2]<0 or $lt[2]>23;
 	croak "invalid minute $lt[1]" if $lt[1]<0 or $lt[1]>59;
 	croak "invalid second $lt[0]"
-		if ($lt[0]<0 or $lt[0]>60 or ($lt[0]==60 and $lt[1]!=59));
+		if ($lt[0]<0 or $lt[0]>=61 or ($lt[0]>=60 and $lt[1]!=59));
 	1;
 }
 
@@ -98,7 +118,7 @@ sub posixtime {
 	my @lt = ($_[0] =~ m{^(\d{4})(-\d{1,2}|\d{2})(-\d{1,2}|\d{2})T?
 			     \s*(?:(\d{1,2})
 				     (?::(\d{2})
-					     (?::(\d{2}))?
+					     (?::(\d{2}(?:\.\d+)?))?
 				     )?
 			     )?$}x)
 		or croak "bad timestamp '$_[0]'";
@@ -107,7 +127,7 @@ sub posixtime {
 	$lt[0]-=1900;
 	$lt[1]--;
 	$_ ||= 0 for (@lt[3..5]);
-	reverse @lt;
+	reverse(@lt);
 }
 
 coerce Timestamp
@@ -135,7 +155,8 @@ as well.
 
 =head2 posixtime()
 
-Alias for the built-in C<localtime>
+Alias for the built-in C<localtime>; this will not return a hi-res
+time unless one is passed in.
 
 =head2 posixtime(Timestamp)
 
@@ -167,19 +188,23 @@ by using the C<coerce =E<gt> 1> flag on a Moose attribute declaration:
 
   package Widget;
   use MooseX::Timestamp;
-  has 'created' =>
+  has 'created' => (
           isa => Timestamp,
           is => "rw",
-          coerce => 1;
+          coerce => 1,
+          );
 
   package main;
   my $widget = new Widget;
   $widget->created("2007-12-07");
   print $widget->created;  # 2007-12-07 00:00:00
 
-With the above, if you set C<created> to a value such as
-automatically get converted into a TimestampTZ in the current time
-zone.
+With the above, if you set C<created> to a value such as automatically
+get converted into a Timestamp in the current time zone.
+
+Timestamps may contain fractional components, but the results of
+conversions from floating point are truncated at the microsecond
+level.
 
 =head2 EXPORTS
 
